group_template_tag_prefix = 'DRKNS_GROUP_TEMPLATE_'  # BEGIN / END
unit_template_tag_prefix = 'DRKNS_UNIT_TEMPLATE_'  # BEGIN / END

groups_tag = 'DRKNS_GROUPS'
group_units_tag = 'DRKNS_GROUP_UNITS'

group_name_tag = 'DRKNS_GROUP_NAME'
unit_name_tag = 'DRKNS_UNIT_NAME'

dependency_groups_names_tag = 'DRKNS_DEPENDENCY_GROUPS_NAMES'
all_group_names_tag = 'DRKNS_ALL_GROUPS_NAMES'
